#include <avr/io.h>
#include <stdio.h>


void USART_init(void)
 { // Initialisieren des USART (Atmel AVR ATmega8515)

    DDRD  &= 0xFE; // PD0 (RXD) auf Input
    DDRD  |= 0x02; // PD1 (TXD) auf Output
    PORTD &= 0xFC; // Pullup-Widerstaende fuer PD0 und PD1 aus.

    UCSRA &= 0xFC; // U2X = 0 --> Doppelte USART Transmission Speed aus
                   // MPCM = 0 --> Multiprozessor Modus aus

    UCSRB  = 0x18; // Alle USART Interrupts aus, 
                   // Receiver und Transmitter einschalten

    UCSRC |= 0x80; // Register UCSRC selektieren fuers Lesen/Schreiben
    UCSRC  = 0x86; // USART auf Asynchron-Modus stellen,
                   // Parity = None, 1 Stopbit,
                   // 8 Datenbits (UCSZ2 = 0 (UCSRB), UCSZ1 = 1, UCSZ0 = 1)

    UBRRH &= 0x7F; // Register UBRRH selektieren fuers Lesen/Schreiben
    UBRRH  = 0x0;  // Baud Rate Generator High Register: Wert 0

    UBRRL  = 25;   // Baud Rate Generator Low Register: Wert 25dez
                   // Bei fosc = 4 MHz ergibt sich dabei eine Baudrate von:
                   // BAUD = ( fosc / (16 * (UBRR+1)) )
                   // --> 9615 BPS (Fehler: 0,2%)
 }



/* -------------------------------------------------------------------------
 * delay_ms()
 *
 * PRE:  4.0000 MHz XTal is assumed
 * POST: busy-wait of msec milliseconds (if function is not interrupted)       
 * -------------------------------------------------------------------------
*/ 
 void delay_ms(int msec)
  {//"volatile" ist noetig, damit nichts wegoptimiert wird !!!
   volatile int i,j;
   for (i=0; i<msec; i++)
    {
     for (j=0; j<185; j++)
       {
        asm ("nop"); 
       }
    }
  }



void USART_Transmit( unsigned char data )
{
/* Wait for empty transmit buffer */
while ( !( UCSRA & (1<<UDRE)) )
;
/* Put data into buffer, sends the data */
UDR = data;
}


int main(void)
 {       
    USART_init(); // USART initialisieren

    printf("### USART-Test-Routine ###\n");
    printf("ATmega8515 - USART-Test !!!\n");
    printf("##########################\n");
    
    while(1)
     {       
       USART_Transmit('!');
       delay_ms(500);
     }

    return 0;
 }
